#!/usr/local/bin/perl

# Parse a refer file, generate a file of RFC-1357 records.
# Jim Davis 26 July 94

# Bugs:  is not strict enough.  Should ensure that all lines
# are <= 79 chars, that only printable ascii is used, that
# forbidden chars (eg. tab, backspace) do not appear.

# get directory where this file is.  
{$0 =~ /^(.*)\/.*$/;  $my_dir = $1; 
 if ($my_dir !~ ?^/?) {$my_dir = $ENV{PWD} . "/" . $my_dir;}
 if ($my_dir =~ ?/$?) {chop ($my_dir);}}
push(@INC, $my_dir);


# each site needs a routine to translate from local numbering system to
# a document identifier.
require "canonical-number.pl" || die "Could not load canonical-number.pl";

# print dates in RFC 1357 standard format.
require "rfc-1357-date.pl" || die "Could not load rfc-1357-date.pl";


# Parse REFER files, return a list of records.
# Each record contains author, title, number, date, and abstract since
# this is the only info in RFC 1357 that I care about.
# Assumptions:
#  A blank line delimits a refer record.
#  A refer file might contain a "record" that really is a comment, but such
#    records will not have TR number.
sub parse_refer {
    local ($file) = @_;		# input file name
    local ($nline) = 0;		# line counter for error messages

    local ($author, $title, $number, $date, $abtract);
    local ($nchar, $ndata);
    local ($char, $data, $line);

    local (@Records) = ();	# list of records

    open (IN, $file) || die "Could not open refer file $file\nStopped";
    while (<IN>) {
	$nline++;

	if (/^$/) {
	    # blank line, finish record.  Add it to list but only if it has
	    # a number.  Otherwise it's probably just a comment.
	    if ($number ne "") {
		push (@Records, join ($;, $author, $title, $number, $date, $abstract));}

	    # start a new record
	    $author = "";
	    $title = "";
	    $number = "";
	    $date = "";
	    $abstract = "";

	    $char = ""; $data = "";
	}
	elsif (/^%(.) (.*)$/) {
	    # It is a tag.  Get the new character and the data on the line
	    $nchar = $1;
	    $ndata = $2;

	    # finish the old tag, if any
	    if ($char eq "") {
		# no prev tag, this must be first one.
	    }
	    elsif ($char eq "A") { # Author
		# This tag can occur many times.  Collect all authors, joined
		# by colon.
		&string_push (*author, &canonicalize_name ($data));
	    }
	    elsif ($char eq "C") { # City
		# ignore
	    }
	    elsif ($char eq "D") { # Date
		$date = $data;
	    }
	    elsif ($char eq "F") { # footnote (or file)
		# ignore
	    }
	    elsif ($char eq "I") { # issuer
		# ignore
	    }
	    elsif ($char eq "J") { # Journal
		# ignore
	    }
	    elsif ($char eq "K") { # keywords
		# ignore
	    }
	    elsif ($char eq "R") { # report number
		$number = &canonicalize_report_number($data);
	    }
	    elsif ($char eq "T") { # Title
		$title = $data;
	    }
	    elsif ($char eq "X") { # Abstract
		$abstract = $data;
	    }
	    else {
		print STDERR "Unknown tag type $char on line $line\n";}

	    # Now start the new tag
	    $char = $nchar;
	    $data = $ndata;
	    $line = $nline;
	}
	else {
	    # continuation line
	    if (/^\.pp/) {next;}
	    chop;
	    $data .= "\n" . $_;}
    }
    close (IN);
    @Records;}


# output a name in the standard format, e.g. "Davis, James R."
#  easy cases:
#   Davis, James R.
#   James R. Davis
#  Harder cases:
#   James R. Davis, Jr.
#   James R. Davis, III
#   J\"{a}mes R. Davis

sub canonicalize_name {
    local ($name) = @_;
    if ($name =~ /^\S+, /) {
	# looks like it is already in canonical form
	$name;}

    else {
	local ($suffix) = "";
	# strip off suffix
	if ($name =~ /,( Jr.)$/) {
	    $suffix = $1;
	    $name = $`;}
	if ($name =~ /,( III)$/) {
	    $suffix = $1;
	    $name = $`;}
	if ($name =~ /,( IV)$/) {
	    $suffix = $1;
	    $name = $`;}
	
	$name =~ / *(\S+)$/;
	
	$1 . ", " . $` . $suffix;
    }}


## print the records in RFC 1357 standard format.

$today = &rfc1357'date_string (time);

sub print_records {
    foreach $record (@_) {
	&print_rfc1357 ($record);
	print "\n";}}

sub print_rfc1357 {
    local ($record) = @_;
    local ($author, $title, $number, $date, $abstract) = 
	split ($;, $record);

    # These three fields must appear in this order.
    print "BIB-VERSION:: CS-TR-v2.0\n";
    print "ID:: $number\n";
    print "ENTRY:: $today\n";

    # these may appear in any order.
    print "TITLE:: $title \n";
    foreach $name (split (":", $author)) {
	print "AUTHOR:: $name\n";}
    print "DATE:: $date\n";
    print "ABSTRACT:: $abstract\n";

    print "END:: $number\n";}

## Utilities

# Like push, but to a string instead of a list (or array).
# uses colon as delimiter
sub string_push {
    local (*string, $data) = @_;

    if ($string eq "") {$string = $data;}
    else {$string .= ":" . $data;}
    $string;}


## Parse command line arguments.
$file = "";

while ($#ARGV >=0) {
    $arg = shift;
    if ($arg =~ /^-./) {
	if (0) {}
	else {
	    die "Unrecognized switch $arg.\nStopped";}}
    else {
	$file = $arg;
    }}

if ($file eq "") {
    die "Missing argument (refer file)\n";}

&print_records (&parse_refer ($file));

